@@ -2,6 +2,7 @@ import asyncio
 import datetime
 import logging
 import logging.handlers
+import json
 import os
 import pprint
 import queue
@@ -14,14 +15,38 @@ import urllib.parse
 
 from .config import Config
 from .timer import Timer
+from .utils import running_in_container
+
 
-# Non-error/warning type of message that is visible without -v flag
 LOG_NOTICE = 25
+"""
+Info log level that is visible in non-verbose mode. It should not be used for warnings and errors.
+"""
+
 logging.addLevelName(LOG_NOTICE, "NOTICE")
 
+L = logging.getLogger(__name__)
+
+_NAME_TO_LEVEL = {
+	"NOTSET": logging.NOTSET,
+	"NOT SET": logging.NOTSET,
+	"NOT_SET": logging.NOTSET,
+	"DEBUG": logging.DEBUG,
+	"INFO": logging.INFO,
+	"NOTICE": LOG_NOTICE,
+	"LOG_NOTICE": LOG_NOTICE,
+	"LOG NOTICE": LOG_NOTICE,
+	"WARNING": logging.WARNING,
+	"WARN": logging.WARNING,
+	"ERROR": logging.ERROR,
+	"FATAL": logging.CRITICAL,
+	"CRITICAL": logging.CRITICAL,
+}
+
 
 class Logging(object):
 
+
 	def __init__(self, app):
 		self.RootLogger = logging.getLogger()
 
@@ -31,23 +56,20 @@ class Logging(object):
 
 		if not self.RootLogger.hasHandlers():
 
-			# Add console logger
-			# Don't initialize this when not on console
-			if os.isatty(sys.stdin.fileno()) or os.environ.get('ASABFORCECONSOLE', '0') != '0':
-				self.ConsoleHandler = logging.StreamHandler(stream=sys.stderr)
-				self.ConsoleHandler.setFormatter(StructuredDataFormatter(
-					fmt=Config["logging:console"]["format"],
-					datefmt=Config["logging:console"]["datefmt"],
-					sd_id=Config["logging"]["sd_id"],
-				))
-				self.ConsoleHandler.setLevel(logging.DEBUG)
-				self.RootLogger.addHandler(self.ConsoleHandler)
+			# Add console logger if needed
+			if os.isatty(sys.stdout.fileno()) or os.environ.get('ASABFORCECONSOLE', '0') != '0' or Config.getboolean("logging:console", "enabled"):
+				self._configure_console_logging()
 
 			# Initialize file handler
 			file_path = Config["logging:file"]["path"]
 
 			if len(file_path) > 0:
 
+				# Ensure file path
+				directory = os.path.dirname(file_path)
+				if not os.path.exists(directory):
+					os.makedirs(directory)
+
 				self.FileHandler = logging.handlers.RotatingFileHandler(
 					file_path,
 					backupCount=Config.getint("logging:file", "backup_count"),
@@ -83,7 +105,7 @@ class Logging(object):
 							async def schedule(app, interval):
 								self.LogRotatingTime = Timer(app, self._on_tick_rotate_check, autorestart=True)
 								self.LogRotatingTime.start(i)
-							asyncio.ensure_future(schedule(app, i), loop=app.Loop)
+							asyncio.ensure_future(schedule(app, i))
 
 					else:
 						self.RootLogger.error("Invalid 'rotate_every' configuration value.")
@@ -106,10 +128,10 @@ class Logging(object):
 						))
 
 					elif url.scheme == 'udp':
-						self.SyslogHandler = AsyncIOHandler(app.Loop, socket.AF_INET, socket.SOCK_DGRAM, (
+						self.SyslogHandler = FormatingDatagramHandler(
 							url.hostname if url.hostname is not None else 'localhost',
 							url.port if url.port is not None else logging.handlers.SYSLOG_UDP_PORT
-						))
+						)
 
 					elif url.scheme == 'unix-connect':
 						self.SyslogHandler = AsyncIOHandler(app.Loop, socket.AF_UNIX, socket.SOCK_STREAM, url.path)
@@ -128,27 +150,49 @@ class Logging(object):
 						self.SyslogHandler.setFormatter(MacOSXSyslogFormatter(sd_id=Config["logging"]["sd_id"]))
 					elif format == '5':
 						self.SyslogHandler.setFormatter(SyslogRFC5424Formatter(sd_id=Config["logging"]["sd_id"]))
+					elif format == '5micro':
+						self.SyslogHandler.setFormatter(SyslogRFC5424microFormatter(sd_id=Config["logging"]["sd_id"]))
+					elif format == 'json':
+						self.SyslogHandler.setFormatter(JSONFormatter())
 					else:
 						self.SyslogHandler.setFormatter(SyslogRFC3164Formatter(sd_id=Config["logging"]["sd_id"]))
 					self.RootLogger.addHandler(self.SyslogHandler)
 
+			# No logging is configured
+			if self.ConsoleHandler is None and self.FileHandler is None and self.SyslogHandler is None:
+				# Let's check if we run in Docker and if so, then log on stderr
+				if running_in_container():
+					self._configure_console_logging()
+
 		else:
 			self.RootLogger.warning("Logging seems to be already configured. Proceed with caution.")
 
 		if Config["logging"].getboolean("verbose"):
 			self.RootLogger.setLevel(logging.DEBUG)
 		else:
-			self.RootLogger.setLevel(Config["logging"]["level"])
+			level_name = Config["logging"]["level"].upper()
+			try:
+				self.RootLogger.setLevel(_NAME_TO_LEVEL.get(level_name, level_name))
+			except ValueError:
+				L.error("Cannot detect logging level '{}'".format(level_name))
 
 		# Fine-grained log level configurations
 		levels = Config["logging"].get('levels')
-		for levelconf in levels.split('\n'):
-			levelconf = levelconf.strip()
-			if len(levelconf) == 0 or levelconf.startswith('#') or levelconf.startswith(';'):
+		for level_line in levels.split('\n'):
+			level_line = level_line.strip()
+			if len(level_line) == 0 or level_line.startswith('#') or level_line.startswith(';'):
+				# line starts with a comment
 				continue
-			loggername, levelname = levelconf.split(' ', 1)
-			level = logging.getLevelName(levelname.upper())
-			logging.getLogger(loggername).setLevel(level)
+			try:
+				logger_name, level_name = level_line.split(' ', 1)
+			except ValueError:
+				L.error("Cannot read line '{}' in '[logging] levels' section, expected format: 'logger_name level_name'.".format(level_line))
+				continue
+			level = _NAME_TO_LEVEL.get(level_name.upper(), level_name.upper())
+			try:
+				logging.getLogger(logger_name).setLevel(level)
+			except ValueError:
+				L.error("Cannot detect logging level '{}' for {} logger".format(level_name, logger_name))
 
 	def rotate(self):
 		if self.FileHandler is not None:
@@ -162,6 +206,29 @@ class Logging(object):
 				self.rotate()
 
 
+	def _configure_console_logging(self):
+		self.ConsoleHandler = logging.StreamHandler(stream=sys.stderr)
+
+		# Disable colors when running in container
+		if running_in_container():
+			self.ConsoleHandler.setFormatter(StructuredDataFormatter(
+				fmt=Config["logging:console"]["format"],
+				datefmt=Config["logging:console"]["datefmt"],
+				sd_id=Config["logging"]["sd_id"],
+				use_color=False
+			))
+		else:
+			self.ConsoleHandler.setFormatter(StructuredDataFormatter(
+				fmt=Config["logging:console"]["format"],
+				datefmt=Config["logging:console"]["datefmt"],
+				sd_id=Config["logging"]["sd_id"],
+				use_color=True
+			))
+
+		self.ConsoleHandler.setLevel(logging.DEBUG)
+		self.RootLogger.addHandler(self.ConsoleHandler)
+
+
 class _StructuredDataLogger(logging.Logger):
 	'''
 This class extends a default python logger class, specifically by adding ``struct_data`` parameter to logging functions.
@@ -186,37 +253,53 @@ class StructuredDataFormatter(logging.Formatter):
 	'''
 
 	empty_sd = ""
+	BLACK, RED, GREEN, YELLOW, BLUE, MAGENTA, CYAN, WHITE = range(8)
 
-	def __init__(self, facility=16, fmt=None, datefmt=None, style='%', sd_id='sd'):
+
+	def __init__(self, facility=16, fmt=None, datefmt=None, style='%', sd_id='sd', use_color: bool = False):
+		# Because of custom formatting, the style is set to percent style and cannot be changed.
+		style = '%'
 		super().__init__(fmt, datefmt, style)
 		self.SD_id = sd_id
 		self.Facility = facility
+		self.UseColor = use_color
 
-
-	def format(self, record):
-		'''
-		Format the specified record as text.
-		'''
-
-		record.struct_data = self.render_struct_data(record.__dict__.get("_struct_data"))
+	def formatMessage(self, record):
+		values = record.__dict__.copy()
+		values["struct_data"] = self.render_struct_data(values.get("_struct_data"))
 
 		# The Priority value is calculated by first multiplying the Facility number by 8 and then adding the numerical value of the Severity.
-		if record.levelno > logging.DEBUG and record.levelno <= logging.INFO:
+		if record.levelno <= logging.DEBUG:
+			severity = 7  # Debug
+			color = self.BLUE
+		elif record.levelno <= logging.INFO:
 			severity = 6  # Informational
+			color = self.GREEN
 		elif record.levelno <= LOG_NOTICE:
 			severity = 5  # Notice
+			color = self.CYAN
 		elif record.levelno <= logging.WARNING:
 			severity = 4  # Warning
+			color = self.YELLOW
 		elif record.levelno <= logging.ERROR:
 			severity = 3  # Error
+			color = self.RED
 		elif record.levelno <= logging.CRITICAL:
 			severity = 2  # Critical
+			color = self.MAGENTA
 		else:
 			severity = 1  # Alert
+			color = self.WHITE
 
-		record.priority = (self.Facility << 3) + severity
-		return super().format(record)
+		if self.UseColor:
+			levelname = record.levelname
+			levelname_color = _COLOR_SEQ % (30 + color) + levelname + _RESET_SEQ
+			values["levelname"] = levelname_color
 
+		values["priority"] = (self.Facility << 3) + severity
+
+		# We use percent style formatting only
+		return self._fmt % values
 
 	def formatTime(self, record, datefmt=None):
 		'''
@@ -235,6 +318,7 @@ class StructuredDataFormatter(logging.Formatter):
 			print("ERROR when logging: {}".format(e), file=sys.stderr)
 			return str(ct)
 
+
 	def render_struct_data(self, struct_data):
 		'''
 		Return the string with structured data.
@@ -242,6 +326,7 @@ class StructuredDataFormatter(logging.Formatter):
 
 		if struct_data is None:
 			return self.empty_sd
+
 		else:
 			return "[{sd_id} {sd_params}] ".format(
 				sd_id=self.SD_id,
@@ -289,11 +374,11 @@ class MacOSXSyslogFormatter(StructuredDataFormatter):
 
 class SyslogRFC3164Formatter(StructuredDataFormatter):
 	"""
-	It implements Syslog formatting for Mac OSX syslog (aka format ``3``).
+	Implementation of a legacy or BSD Syslog (RFC 3164) formatting (aka format ``3``).
 	"""
 
 	def __init__(self, fmt=None, datefmt=None, style='%', sd_id='sd'):
-		fmt = '<%(priority)s>%(asctime)s {app_name} {proc_id} %(levelname)s %(name)s %(struct_data)s%(message)s\000'.format(
+		fmt = '<%(priority)s>%(asctime)s {hostname} {app_name}[{proc_id}]:%(levelname)s %(name)s %(struct_data)s%(message)s\000'.format(
 			app_name=Config["logging"]["app_name"],
 			hostname=socket.gethostname(),
 			proc_id=os.getpid(),
@@ -324,12 +409,75 @@ class SyslogRFC5424Formatter(StructuredDataFormatter):
 		self.converter = time.gmtime
 
 
-class AsyncIOHandler(logging.Handler):
+class SyslogRFC5424microFormatter(StructuredDataFormatter):
+	"""
+	It implements Syslog formatting for syslog (aka format ``micro``) in RFC5424micro format.
+	"""
 
-	'''
-A logging handler similar to a standard ``logging.handlers.SocketHandler`` that utilizes ``asyncio``.
-It implements a queue for decoupling logging from a networking. The networking is fully event-driven via ``asyncio`` mechanisms.
-	'''
+	empty_sd = "-"
+
+	def __init__(self, fmt=None, datefmt=None, style='%', sd_id='sd'):
+		fmt = '<%(priority)s>1 %(asctime)sZ {hostname} {app_name} {proc_id} %(name)s [log l="%(levelname)s"]%(struct_data)s%(message)s'.format(
+			app_name=Config["logging"]["app_name"],
+			hostname=socket.gethostname(),
+			proc_id=os.getpid(),
+		)
+
+		super().__init__(fmt=fmt, datefmt='%Y-%m-%dT%H:%M:%S.%f', style=style, sd_id=sd_id)
+		self.converter = time.gmtime
+
+
+class JSONFormatter(logging.Formatter):
+
+	def __init__(self):
+		self.Enricher = {}
+		instance_id = os.environ.get("INSTANCE_ID")
+		service_id = os.environ.get("SERVICE_ID")
+		node_id = os.environ.get("NODE_ID")
+		hostname = socket.gethostname()
+		if instance_id is not None:
+			self.Enricher["instance_id"] = instance_id
+		if service_id is not None:
+			self.Enricher["service_id"] = service_id
+		if node_id is not None:
+			self.Enricher["node_id"] = node_id
+		if hostname is not None:
+			self.Enricher["hostname"] = hostname
+
+	def _default(self, obj):
+		# If obj is not json serializable, convert it to string
+		try:
+			return str(obj)
+		except Exception:
+			raise TypeError("Error when logging. Object {} of type {} is not JSON serializable.".format(obj, type(obj)))
+
+	def format(self, record):
+		r_copy = record.__dict__.copy()
+		r_copy.update(self.Enricher)
+		return json.dumps(r_copy, default=self._default)
+
+
+class FormatingDatagramHandler(logging.handlers.DatagramHandler):
+
+	def __init__(self, host, port):
+		super().__init__(host, port)
+
+	def emit(self, record):
+		"""
+		Add formatting to DatagramHandler. See https://docs.python.org/3/library/logging.handlers.html
+		"""
+		try:
+			msg = self.format(record).encode('utf-8')
+			self.send(msg)
+		except Exception:
+			self.handleError(record)
+
+
+class AsyncIOHandler(logging.Handler):
+	"""
+	A logging handler similar to a standard `logging.handlers.SocketHandler` that utilizes `asyncio`.
+	It implements a queue for decoupling logging from a networking. The networking is fully event-driven via `asyncio` mechanisms.
+	"""
 
 	def __init__(self, loop, family, sock_type, address, facility=logging.handlers.SysLogHandler.LOG_LOCAL1):
 		logging.Handler.__init__(self)
@@ -370,16 +518,23 @@ It implements a queue for decoupling logging from a networking. The networking i
 		self._loop.add_writer(self._socket, self._on_write)
 		self._loop.add_reader(self._socket, self._on_read)
 
-
 	def _on_write(self):
 		self._write_ready = True
 		self._loop.remove_writer(self._socket)
 
 		while not self._queue.empty():
-			# TODO: Handle eventual error in writing -> break the cycle and restart on write handler
 			msg = self._queue.get_nowait()
-			self._socket.sendall(msg)
-
+			try:
+				self._socket.sendall(msg)
+			except Exception as e:
+				# Contingency dump when the socket is not ready
+				print(msg.decode("utf-8"), file=sys.stderr)
+				print(
+					"Error when writing to syslog '{}': {}".format(self._address, e),
+					traceback.format_exc(),
+					sep="\n",
+					file=sys.stderr
+				)
 
 	def _on_read(self):
 		try:
@@ -389,14 +544,11 @@ It implements a queue for decoupling logging from a networking. The networking i
 		except Exception as e:
 			print("Error on the syslog socket '{}'".format(self._address), e, file=sys.stderr)
 
-		# Close a socket - there is no reason for reading or socket is actually closed
-		self._reset()
-
 
 	def emit(self, record):
-		'''
+		"""
 		This is the entry point for log entries.
-		'''
+		"""
 		try:
 			msg = self.format(record).encode('utf-8')
 
@@ -408,7 +560,7 @@ It implements a queue for decoupling logging from a networking. The networking i
 					self._enqueue(msg)
 
 			else:
-				self._enqueue(record)
+				self._enqueue(msg)
 
 
 		except Exception as e:
@@ -418,3 +570,8 @@ It implements a queue for decoupling logging from a networking. The networking i
 
 	def _enqueue(self, record):
 		self._queue.put(record)
+
+
+_RESET_SEQ = "\033[0m"
+_COLOR_SEQ = "\033[1;%dm"
+_BOLD_SEQ = "\033[1m"
