@@ -12,7 +12,7 @@ class Module(asab.Module):
 
 	def __init__(self, app):
 		super().__init__(app)
-		sttype = asab.Config.get('asab:storage', 'type')
+		sttype = asab.Config.get('asab:storage', 'type', fallback=None)
 
 		if sttype == 'inmemory':
 			from .inmemory import StorageService
@@ -26,5 +26,10 @@ class Module(asab.Module):
 			from .elasticsearch import StorageService
 			self.Service = StorageService(app, "asab.StorageService")
 
+		elif sttype is None:
+			L.critical("Missing configuration for [asab:storage] type.")
+			raise SystemExit("Exit due to a critical configuration error.")
+
 		else:
-			L.error("Unknown asab:storage type '{}'".format(sttype))
+			L.critical("Unknown configuration type '{}' in [asab:storage].".format(sttype))
+			raise SystemExit("Exit due to a critical configuration error.")
