@@ -1,10 +1,17 @@
 import aiohttp.abc
 
 from ..log import LOG_NOTICE
+from .. import Config
 
 
 class AccessLogger(aiohttp.abc.AbstractAccessLogger):
 
+	def __init__(self, logger, log_format) -> None:
+		super().__init__(logger, log_format)
+		self.App = logger.App
+		self.WebService = self.App.get_service("asab.WebService")
+		self.web_metrics_config = Config.getboolean("asab:metrics", "web_requests_metrics", fallback=False)
+
 	def log(self, request, response, time):
 		struct_data = {
 			'I': request.remote,
@@ -34,3 +41,11 @@ class AccessLogger(aiohttp.abc.AbstractAccessLogger):
 			struct_data['Ix'] = xfwd[:128]
 
 		self.logger.log(LOG_NOTICE, '', struct_data=struct_data)
+
+		# Metrics
+		if self.web_metrics_config:
+			path = request.match_info.get_info().get("formatter")
+			if path is None:
+				path = request.path
+
+			self.WebService.WebRequestsMetrics.set_metrics(time, request.method, path, str(response.status))
