@@ -1,10 +1,16 @@
 import json
-import logging
 import uuid
+import logging
 import datetime
+import functools
+import dataclasses
+
+
 import aiohttp.web
 import fastjsonschema
 
+from ... import exceptions
+
 #
 
 L = logging.getLogger(__name__)
@@ -13,6 +19,21 @@ Lex = logging.getLogger("asab.web")
 
 #
 
+_no_log_headers = {
+	"Authorization",
+}
+
+
+def _filter_logging_headers(headers):
+	"""
+	Filter HTTP headers to include in log messages.
+	"""
+	return {
+		k: v for k, v in headers.items()
+		if k not in _no_log_headers
+	}
+
+
 class JSONDumper(object):
 
 	def __init__(self, pretty):
@@ -30,11 +51,23 @@ class JSONDumper(object):
 			return m()
 
 		if isinstance(o, datetime.datetime):
-			return o.isoformat()
+			if o.tzinfo == datetime.timezone.utc:
+				# isoformat ends with "+00:00", replace it with "Z"
+				return o.isoformat()[:-6] + "Z"
+			elif o.tzinfo is not None:
+				# The datetime object is timezone-aware but not UTC
+				# NOT WANTED, using non-UTC timestamps is not recommended
+				return o.isoformat()
+			else:
+				# The datetime object is timezone-naive -> interpret it as UTC
+				return o.isoformat() + "Z"
 
 		elif isinstance(o, bytes):
 			return o.hex()
 
+		elif dataclasses.is_dataclass(o):
+			return dataclasses.asdict(o)
+
 		try:
 			return json.JSONEncoder.default(self, o)
 		except TypeError:
@@ -73,57 +106,133 @@ async def JsonExceptionMiddleware(request, handler):
 
 	# HTTP errors to JSON
 	except aiohttp.web.HTTPError as ex:
+		if ex.status == 401:
+			result = "NOT-AUTHORIZED"
+		elif ex.status == 404:
+			result = "NOT-FOUND"
+		else:
+			result = "ERROR"
+
 		respdict = {
-			'status': ex.status,
-			'message': ex.text[5:]
+			'result': result,
+			'message': ex.text,
 		}
 		if ex.status >= 400:
 			euuid = uuid.uuid4()
-			struct_data = {'uuid': str(euuid)}
-			respdict['uuid'] = str(euuid)
-			struct_data.update(request.headers)
-			struct_data['path'] = request.path
-			struct_data['status'] = ex.status
+			respdict["uuid"] = str(euuid)
+			struct_data = {
+				"uuid": str(euuid),
+				"path": request.path,
+				"status": ex.status,
+				**_filter_logging_headers(request.headers)
+			}
 			Lex.error(ex, struct_data=struct_data)
 
 		ex.content_type = 'application/json'
-		ex.text = json.dumps(respdict, indent=4)
+		ex.text = json.dumps(respdict)
 		raise ex
 
 	# KeyError translates to 404
 	except KeyError as e:
 		euuid = uuid.uuid4()
-		Lex.warning("KeyError when handling web request", exc_info=e, struct_data={'uuid': str(euuid)})
+		struct_data = {
+			"uuid": str(euuid),
+			"path": request.path,
+			"status": 404,
+			**_filter_logging_headers(request.headers)
+		}
+		Lex.warning("KeyError when handling web request", exc_info=True, struct_data=struct_data)
 
 		if len(e.args) > 1:
-			message = e.args[0] % e.args[1:]
-		elif e.args[0] is None:
-			message = "KeyError"
-		else:
+			message = e.args[0].format(*e.args[1:])
+		elif len(e.args) == 1 and e.args[0] is not None:
 			message = e.args[0]
+		else:
+			message = "KeyError"
 
-		return aiohttp.web.Response(
-			text=json.dumps({
-				"status": 404,
+		return json_response(
+			request,
+			data={
+				"result": "NOT-FOUND",
 				"message": message,
 				"uuid": str(euuid),
-			}, indent=4),
+			},
 			status=404,
-			content_type='application/json'
 		)
 
-	# Other errors to JSON
-	except Exception as e:
+	# ValidationError translates to 400
+	except exceptions.ValidationError as e:
 		euuid = uuid.uuid4()
-		Lex.exception("Exception when handling web request", exc_info=e, struct_data={'uuid': str(euuid)})
-		return aiohttp.web.Response(
-			text=json.dumps({
-				"status": 500,
+		struct_data = {
+			"uuid": str(euuid),
+			"path": request.path,
+			"status": 400,
+			**_filter_logging_headers(request.headers)
+		}
+		Lex.warning("ValidationError when handling web request", exc_info=True, struct_data=struct_data)
+
+		if len(e.args) > 1:
+			message = e.args[0].format(*e.args[1:])
+		elif len(e.args) == 1 and e.args[0] is not None:
+			message = e.args[0]
+		else:
+			message = "ValidationError"
+		return json_response(
+			request,
+			data={
+				"result": "VALIDATION-ERROR",
+				"message": message,
+				"uuid": str(euuid),
+			},
+			status=400,
+		)
+
+	# Conflict surfaces as HTTP 409 Conflict
+	except exceptions.Conflict as e:
+		euuid = uuid.uuid4()
+		struct_data = {
+			"uuid": str(euuid),
+			"path": request.path,
+			"status": 409,
+			**_filter_logging_headers(request.headers)
+		}
+		Lex.warning("Conflict when handling web request", exc_info=True, struct_data=struct_data)
+
+		if len(e.args) > 1:
+			message = e.args[0].format(*e.args[1:])
+		elif len(e.args) == 1 and e.args[0] is not None:
+			message = e.args[0]
+		else:
+			message = "Conflict"
+
+		return json_response(
+			request,
+			data={
+				"result": "CONFLICT",
+				"message": message,
+				"uuid": str(euuid),
+			},
+			status=409,
+		)
+
+	# Other errors surface as HTTP 500 Internal server error
+	except Exception:
+		euuid = uuid.uuid4()
+		struct_data = {
+			"uuid": str(euuid),
+			"path": request.path,
+			"status": 500,
+			**_filter_logging_headers(request.headers)
+		}
+		Lex.exception("Exception when handling web request", exc_info=True, struct_data=struct_data)
+		return json_response(
+			request,
+			data={
+				"result": "ERROR",
 				"message": "Internal Server Error",
 				"uuid": str(euuid),
-			}, indent=4),
+			},
 			status=500,
-			content_type='application/json'
 		)
 
 
@@ -174,31 +283,36 @@ def json_schema_handler(json_schema, *_args, **_kwargs):
 
 		form_content_types = frozenset(['', 'application/x-www-form-urlencoded', 'multipart/form-data'])
 
+		@functools.wraps(func)
 		async def validator(*args, **kwargs):
 			# Initializing fastjsonschema.compile method and generating
 			# the validation function for validating JSON schema
 			request = args[-1]
 			if request.content_type == 'application/json':
-				data = await request.json()
+				try:
+					data = await request.json()
+				except json.decoder.JSONDecodeError:
+					raise aiohttp.web.HTTPBadRequest(reason="Failed to parse JSON request")
 			elif request.content_type in form_content_types:
 				multi_dict = await request.post()
 				data = {k: v for k, v in multi_dict.items()}
 			else:
-				Lex.warning(f"Unsupported content-type {request.content_type} for request {request}")
-				raise aiohttp.web.HTTPBadRequest(reason=f"Unsupported content-type {request.content_type}")
+				raise aiohttp.web.HTTPBadRequest(reason="Unsupported content-type {}".format(request.content_type))
 			# Checking the validation on JSON data set
 			try:
 				validate(data)
 				kwargs['json_data'] = data
 			except fastjsonschema.exceptions.JsonSchemaException as e:
-				Lex.warning(f"Can not validate request {request}. Reason: {e}")
 				raise aiohttp.web.HTTPBadRequest(reason=str(e))
 			except Exception as e:
-				Lex.error(f"Unknown validation error for request {request}. Reason: {e}")
+				Lex.error("JSON validation error. Reason: {}".format(e), struct_data={"path": request.path, "method": request.method})
 				raise e
 
 			return await func(*args, **kwargs)
 
+		# This is here for Swagger documentation
+		setattr(validator, "json_schema", json_schema)
+
 		return validator
 
 	return decorator
