@@ -1,19 +1,17 @@
-import logging
-
-from .service import ZooKeeperService
 from .container import ZooKeeperContainer
-
 from ..abc.module import Module
 
-#
-
-L = logging.getLogger(__name__)
-
-#
-
 
 class Module(Module):
 
 	def __init__(self, app):
 		super().__init__(app)
+
+		from .service import ZooKeeperService
 		self.Service = ZooKeeperService(app, "asab.ZooKeeperService")
+
+
+__all__ = [
+	"ZooKeeperContainer",
+	"Module",
+]
